import { executeComposer } from "./execute.js";
import * as remoteControl from "./remoteControl.js";

/**
 * @typedef Question
 * @property {string} question,
 * @property {"string" | "boolean" | "number"} type
 * @property {unknown} default
 */

/**
 * @typedef {Record<string, Question>} ArgType
 */

/**
 * @template {ArgType} Args
 * @typedef {import("../utils/workspace").Workspace<Args>} Workspace
 */

/**
 * @template {ArgType} Args
 * @typedef  {{
 *   [K in keyof Args]: Args[K]["type"] extends "string" ? string: Args[K]["type"] extends "boolean"? boolean: Args[K]["type"] extends "number"? number: never
 * }} ArgValues
 */

/**
 * @template {ArgType} Args
 * @typedef PackageDefinition
 * @property {string} name
 * @property {string} version
 * @property {boolean} dev
 * @property {ConditionDefinition<Args>} [condition]
 */

/**
 * @template {ArgType} Args
 * @typedef {(options: Workspace<Args>) => boolean} ConditionDefinition
 */

/**
 * @template {ArgType} Args
 * @typedef {import("../files/processors").ScriptFile<Args>
 *  | import("../files/processors").TextFile<Args>
 *  | import("../files/processors").SvelteFile<Args>
 *  | import("../files/processors").JsonFile<Args>
 *  | import("../files/processors").HtmlFile<Args>
 *  | import("../files/processors").CssFile<Args>
 * } FileTypes
 */

/**
 * @typedef WebsiteMetadata
 * @property {string} website.logo
 * @property {string[]} website.keywords
 */

/**
 * @typedef ComposerConfigMetadata
 * @property {string} id
 * @property {string} package
 * @property {string} version
 * @property {string} name
 * @property {string} description
 * @property {WebsiteMetadata} website
 */

/**
 * @template {ArgType} Args
 * @typedef ComposerConfig
 * @property {ComposerConfigMetadata} metadata
 * @property {Args?} options
 * @property {PackageDefinition<Args>[]} packages
 * @property {FileTypes<Args>[]} files
 * @property {(options: Workspace<Args>) => Promise<void>?} [installHook]
 * @property {(options: Workspace<Args>) => Promise<void>?} [uninstallHook]
 */

/**
 * @template {ArgType} Args
 * @param {ComposerConfig<Args>} config
 * @returns {ComposerConfig<Args>}
 */
export function defineComposerConfig(config) {
    return config;
}

/**
 * @template {ArgType} Args
 * @typedef Composer
 * @property {ComposerConfig<Args>} config
 * @property {ComposerTestConfig<Args>} [tests]
 */

/**
 * @template {ArgType} Args
 * @param {ComposerConfig<Args>} config
 * @returns {Composer<Args>}
 */
export function defineComposer(config, tests) {
    const { remoteControlled } = remoteControl.getStatus();
    if (!remoteControlled) {
        executeComposer(config);
    }

    return { config, tests };
}

/**
 * @typedef Tests
 * @property {(selector: string, property: string, expectedValue: string) => Promise<string>} check
 */

/**
 * @typedef TestDefinition
 * @property {string} name
 * @property {(tests: Tests) => void} run
 */

/**
 * @template {ArgType} Args
 * @typedef ComposerTestConfig
 * @property {FileTypes<Args>[]} files
 * @property {TestDefinition[]} tests
 */

/**
 * @template {ArgType} Args
 * @param {ComposerTestConfig<Args>} tests
 * @returns {ComposerTestConfig<Args>}
 */
export function defineComposerTests(tests) {
    return tests;
}
